/* This file is automatically generated by buildconfig */

#ifndef HISMETHODS_H
#define HISMETHODS_H 1

#include "hisinterface.h"

#define NUM_HIS_METHODS 1

extern HIS_METHOD his_methods[NUM_HIS_METHODS];

#endif /* HISMETHODS_H */
