/* This file is automatically generated by buildconfig. */

#include "hisinterface.h"
#include "hismethods.h"
#include "hisv6/hisv6.h"

HIS_METHOD his_methods[1] = {
{
    "hisv6",
    hisv6_open,
    hisv6_close,
    hisv6_sync,
    hisv6_lookup,
    hisv6_check,
    hisv6_write,
    hisv6_replace,
    hisv6_expire,
    hisv6_walk,
    hisv6_remember,
    hisv6_ctl
}
};

